***************************************************************************
* FUNCTION: GRID()
* AUTHOR  : JAKE JACOB
* VERSION : CLIPPER SUMMER '87
*
* COPYRIGHT (C) 1988 NANTUCKET CORP.
*
* PARAMETERS:
* WID              = WIDTH OF THE HILITED BAR CURSOR.
* TOP,LEFT,BOT,RIT = ABSOLUTE LIMITS OF THE CURSOR DOMAIN.
* SROW,SCOL        = STARTING ROW AND COLUMN POSITIONS OF THE CURSOR.
* ROWSKIP          = ROWS TO SKIP FOR EACH VERTICAL CURSOR MOVEMENT.
* COLSKIP          = CURSOR-WIDTHS TO SKIP FOR EACH
*                    HORIZONTAL CURSOR MOVEMENT.
*
* DEFAULT VALUES   = 1,0,0,24,79,TOP,LEFT,1,1
*
* VARIABLES:
* OLDCLRS          = COLOR SETTING OF THE CALLING PROCEDURE.
* HICLRS           = THE HILITE OF THE COLOR SETTING OF THE CALLING
*                    PROCEDURE.
* CRSOR            = THE STRING WHERE THE HILITED CURSOR IS.
* OLDROW,OLDCOL    = OLD POSITION OF THE HILITED CURSOR WITHIN AGRID().
* OLDCRSOR         = THE OLD STRING WHERE THE HILITED CURSOR WAS.
* RETVAL           = RETURN VALUE.
* KEY              = INKEY() VALUE OF THE LAST KEY PRESSED.
* RTRN             = A RETURN FLAG VARIABLE.

FUNCTION GRID
PARAMETERS WID,TOP,LEFT,BOT,RIT,SROW,SCOL,ROWSKIP,COLSKIP

   PRIVATE OLDCLRS,HICLRS,CRSOR,OLDROW,OLDCOL,OLDCRSOR,RETVAL,KEY,RTRN
   SET CURSOR OFF
   RTRN = .F.

   * check if width is passed as a dummy param or 0
   WID = IF( PCOUNT()=0, 1, IF( EMPTY(WID), 1, WID ) )

   * check top
   TOP = IF( PCOUNT()<2, 0, IF( EMPTY(TOP), 0, TOP ) )
   * if top is negative then return a null
   IF TOP < 0
      RTRN = .T.
   ENDIF

   * check left
   LEFT = IF( PCOUNT()<3, 0, IF( EMPTY(LEFT), 0, LEFT ) )
   * if left is negative then return a null
   IF LEFT < 0
      RTRN = .T.
   ENDIF

   * check bottom
   BOT = IF( PCOUNT()<4, 24, IF( EMPTY(BOT), 24, BOT ) )
   * if bottom is less than top or more than 24 then return a null
   IF BOT < TOP .OR. BOT > 24
      RTRN = .T.
   ENDIF

   * check right
   RIT = IF( PCOUNT()<5, 79, IF( EMPTY(RIT), 79, RIT ) )
   * if right is less than left or more than 79 then return a null
   IF RIT < LEFT .OR. RIT > 79
      RTRN = .T.
   ENDIF

   * check starting row
   SROW = IF( PCOUNT()<6, TOP, IF( EMPTY(SROW), TOP, SROW ) )
   * if srow is less than top or more then bottom then return a null
   IF SROW < TOP .OR. SROW > BOT
      RTRN = .T.
   ENDIF

   * check starting column
   SCOL = IF( PCOUNT()<7, LEFT, IF( EMPTY(SCOL), LEFT, SCOL ) )
   * if scol is less than left or more than right then return a null
   IF SCOL < LEFT .OR. SCOL > RIT
      RTRN = .T.
   ENDIF

   * check how many rows to skip for each vertical cursor movement
   ROWSKIP = IF( PCOUNT()<8, 1, IF( EMPTY(ROWSKIP), 1, ROWSKIP ) )

   * check how many widths to skip for each horizontal cursor movement
   COLSKIP = IF( PCOUNT()<9, 1, COLSKIP )

   * save color setting
   OLDCLRS = SETCOLOR()

   * return a null if return flag is true
   IF RTRN
      SETCOLOR(OLDCLRS)
      SET CURSOR ON
      RETURN ""
   ENDIF

   * get the hilite of the color setting
   HICLRS = SUBSTR( OLDCLRS, AT(",", OLDCLRS) + 1 ,;
      AT( ",", SUBSTR( OLDCLRS, AT( ",", OLDCLRS) + 1, LEN(OLDCLRS) ) ) )

   * save the screen to a variable for string operation
   SAVE SCREEN TO SCRN

   CRSOR = ""
   ROW = SROW
   COL = SCOL

   * build the cursor string from the screen variable
   FOR I = 1 TO WID*2 STEP 2
      CRSOR = CRSOR + SUBSTR(SCRN,ROW*160+COL*2+I,1)
   NEXT

   * set the color to the hilite of the old color
   SETCOLOR(HICLRS)

   * paint the cursor string
   @ ROW,COL SAY CRSOR

   KEY = 0

   * start the cursor movement loop
   DO WHILE KEY <> 27 .AND. KEY <> 13

      SETCOLOR(HICLRS)
      OLDROW = ROW
      OLDCOL = COL
      OLDCRSOR = CRSOR

      INKEY(0)         && WAITING BETWEEN MOVEMENTS

      KEY = LASTKEY()  && CAPTURE THE LAST KEY

      * if uparrow then decrement the row
      IF KEY = 5
         ROW = ROW - IF( (ROW-ROWSKIP)<TOP, 0, ROWSKIP )

      * if dnarrow then increment the row
      ELSEIF KEY = 24
         ROW = ROW + IF( (ROW+ROWSKIP)>BOT, 0, ROWSKIP )

      * if leftarrow then decrement the column
      ELSEIF KEY = 19
         COL = COL - IF( (COL-WID*COLSKIP)<LEFT, 0, WID*COLSKIP )

      * if rightarrow then increment the column
      ELSEIF KEY = 4
         COL = COL + IF( (COL+WID*COLSKIP+WID)>RIT, 0, COLSKIP*WID )

      * if homekey then decrement both
      ELSEIF KEY = 1
         ROW = ROW - IF( (ROW-ROWSKIP)<TOP, 0, ROWSKIP )
         COL = COL - IF( (COL-WID*COLSKIP)<LEFT, 0, WID*COLSKIP )

      * if endkey then increment row and decrement column
      ELSEIF KEY = 6
         ROW = ROW + IF( (ROW+ROWSKIP)>BOT, 0, ROWSKIP )
         COL = COL - IF( (COL-WID*COLSKIP)<LEFT, 0, WID*COLSKIP )

      * if pgdn then increment both
      ELSEIF KEY = 3
         ROW = ROW + IF( (ROW+ROWSKIP)>BOT, 0, ROWSKIP )
         COL = COL + IF( (COL+WID*COLSKIP+WID)>RIT, 0, COLSKIP*WID )

      * if pgup then decrement row and increment column
      ELSEIF KEY = 18
         ROW = ROW - IF( (ROW-ROWSKIP)<TOP, 0, ROWSKIP )
         COL = COL + IF( (COL+WID*COLSKIP+WID)>RIT, 0, COLSKIP*WID )

      * if ctrl-home then go to the top-left-most "allowable"
      * position for the cursor which may or may not be the border
      ELSEIF KEY = 29
         ROW = SROW - INT( (SROW-TOP)/ROWSKIP ) * ROWSKIP
         COL = SCOL - INT( (SCOL-LEFT)/(WID*COLSKIP) ) * WID * COLSKIP

      * if ctrl-end then go to the bottom-left-most "allowable"
      * position for the cursor which may or may not be the border
      ELSEIF KEY = 23
         ROW = SROW + INT( (BOT-SROW)/ROWSKIP ) * ROWSKIP
         COL = SCOL - INT( (SCOL-LEFT)/(WID*COLSKIP) ) * WID * COLSKIP

      * if ctrl-pgup then go to the top-right-most "allowable"
      * position for the cursor which may or may not be the border
      ELSEIF KEY = 31
         ROW = SROW - INT( (SROW-TOP)/ROWSKIP ) * ROWSKIP
         COL = SCOL + INT( (RIT-SCOL-WID)/(WID*COLSKIP) ) * WID * COLSKIP

      * if ctrl-pgdn then go to the bottom-right-most "allowable"
      * position for the cursor which may or may not be the border
      ELSEIF KEY = 30
         ROW = SROW + INT( (BOT-SROW)/ROWSKIP ) * ROWSKIP
         COL = SCOL + INT( (RIT-SCOL-WID)/(WID*COLSKIP) ) * WID * COLSKIP

      ENDIF

      * rebuild the cursor string
      CRSOR = ""
      FOR I = 1 TO WID*2 STEP 2
         CRSOR = CRSOR + SUBSTR(SCRN, ROW*160+COL*2+I, 1)
      NEXT

      * repaint the cursor string in the new position
      @ ROW,COL SAY CRSOR

      * repaint the old cursor string in the old position
      * with the old color setting
      IF ROW != OLDROW .OR. COL != OLDCOL
         SETCOLOR(OLDCLRS)
         @ OLDROW,OLDCOL SAY OLDCRSOR
      ENDIF

   ENDDO

   * if esckey then abort the selection and return a null
   * else return the cursor string
   RETVAL = IF( KEY=27, "", CRSOR )

   * reset back to the old colors
   SETCOLOR(OLDCLRS)
   SET CURSOR ON
RETURN RETVAL

* end of code.
